import 'package:flutter/material.dart' as f show MaterialApp, ThemeData;
import 'package:particle_ui/particle.dart';

class App extends StatelessWidget {
  final ThemeSettings theme;
  final String title;
  final Widget home;
  final Map<String, WidgetBuilder> routes;
  final List<SingleChildWidget> providers;

  const App(
      {Key key, this.theme, this.title, this.home, this.routes, this.providers})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    ThemeSettings _theme = _mergeThemeSettings(this.theme);

    Color _bgColor = _theme.fills != null ? _theme.fills[FillType.background] : Color('FFFFFF');

    return Theme(settings: _theme, child: MultiProvider(
      providers: providers,
      child: f.MaterialApp(
          theme: f.ThemeData(
              scaffoldBackgroundColor: _bgColor.to32BitARGBColor(),
              //primaryColor: _theme.colorSettings[ColorOptions.primary].to32BitARGBColor(),
              //accentColor: _theme.colorSettings[ColorOptions.accent].to32BitARGBColor()
          ),
          title: title,
          home: home,
          routes: routes),
    ));
  }

  ThemeSettings _mergeThemeSettings(ThemeSettings theme) {
    ThemeSettings _fallback = ThemeSettings.fallback();

    if (theme != null) {

      Map<FontType, Font> _defaultFonts = _fallback.fonts;
      Map<FontType, Font> _overideFonts = theme.fonts;
      Map<FontType, Font> _newFonts = _mergeMap([_defaultFonts, _overideFonts]);

      Map<IconDataType, IconData> _defaultIconData = _fallback.iconData;
      Map<IconDataType, IconData> _overideIconData = theme.iconData;
      //Map<IconDataType, IconData> _newIconData = [_defaultIconData, _overideIconData].reduce((map1, map2) => map1..addAll(map2));
      Map<IconDataType, IconData> _newIconData = _mergeMap([_defaultIconData, _overideIconData]);

      Map<FillType, Fill> _defaultFills = _fallback.fills;
      Map<FillType, Fill> _overideFills = theme.fills;
      //Map<FillType, Fill> _newFills = [_defaultFills, _overideFills].reduce((map1, map2) => map1..addAll(map2));
      Map<FillType, Fill> _newFills = _mergeMap([_defaultFills, _overideFills]);


      Map<TextType, Text> _defaultTexts = _fallback.texts;
      Map<TextType, Text> _overideTexts = theme.texts;
      //Map<TextType, Text> _newTexts = [_defaultTexts, _overideTexts].reduce((map1, map2) => map1..addAll(map2));
      Map<TextType, Text> _newTexts = _mergeMap([_defaultTexts, _overideTexts]);

      Map<DividerType, Divider> _defaultDividers = _fallback.dividers;
      Map<DividerType, Divider> _overideDividers = theme.dividers;
      //Map<DividerType, Divider> _newDividers = [_defaultDividers, _overideDividers].reduce((map1, map2) => map1..addAll(map2));
      Map<DividerType, Divider> _newDividers = _mergeMap([_defaultDividers, _overideDividers]);

      return ThemeSettings(
        fonts: _newFonts,
        iconData: _newIconData,
        fills: _newFills,
        texts: _newTexts,
        dividers: _newDividers,
      );
    } else {
      return _fallback;
    }
  }

  _copyValues<K, V>(
    Map<K, V> from, Map<K, V> to, bool recursive, bool acceptNull) {
    for (var key in from.keys) {
      if (from[key] is Map<K, V> && recursive) {
        if (!(to[key] is Map<K, V>)) {
          to[key] = <K, V>{} as V;
        }
        _copyValues(from[key] as Map, to[key] as Map, recursive, acceptNull);
      } else {
        if (from[key] != null || acceptNull) to[key] = from[key];
      }
    }
  }

  Map<K, V> _mergeMap<K, V>(Iterable<Map<K, V>> maps,
      {bool recursive: true, bool acceptNull: false}) {
    Map<K, V> result = <K, V>{};
    maps.forEach((Map<K, V> map) {
      if (map != null) _copyValues(map, result, recursive, acceptNull);
    });
    return result;
  }
}
